﻿using System;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;
using System.Text;

namespace SVMctl_I2C_CSharp
{
    /// <summary>
    /// DllImport属性を用いて定義したメソッドを格納するクラス
    /// </summary>
    public static class NativeMethods
    {
        /// <summary>
        /// SVI05API.dllのPath
        /// </summary>
        private const string dllName = "SVI05API.dll";

        /// <summary>
        /// SVI05API API戻り値定義
        /// </summary>
        
        // 正常終了時の戻り値
        public const UInt32 RET_NORMAL = 0x1;

        // エラー発生時の戻り値
        public const UInt32 RET_ERROR_DEVICE = 0xe0000000;
        public const UInt32 RET_ERROR_DEVOPEN = 0xf0000001;
        public const UInt32 RET_ERROR_MULTIOPEN = 0xf0000002;
        public const UInt32 RET_ERROR_NOOPEN = 0xf0000003;
        public const UInt32 RET_ERROR_PARAMETER = 0xf0000004;
        public const UInt32 RET_ERROR_NOIDLE = 0xf0000006;
        public const UInt32 RET_ERROR_SVI = 0xf1000000;

        /// <summary>
        /// SVI05API ステータス定義
        /// </summary>
        public const UInt32 STS_SUCCESS = 0x0001;
        public const UInt32 STS_FRAMEPENDING = 0x0002;
        public const UInt32 STS_CAPCANCELED = 0x0003;
        public const UInt32 STS_BUSY = 0x0011;
        public const UInt32 STS_RECOVERYMODE = 0x0012;
        public const UInt32 STS_I2CACTIVE = 0x0013;
        public const UInt32 STS_CMD_INVALID = 0x0101;
        public const UInt32 STS_PRM_INVALID = 0x0102;
        public const UInt32 STS_SEQ_INVALID = 0x0103;
        public const UInt32 STS_I2C_ACKTIMEOUT = 0x0601;
        public const UInt32 STS_I2C_PRETIMEOUT = 0x0602;
        public const UInt32 STS_I2C_POSTTIMEOUT = 0x0603;
        public const UInt32 STS_INTERNAL_ERROR = 0x0F01;
        public const UInt32 STS_RESOURCE_ERROR = 0x0F02;

        /// <summary>
        /// USB3.0接続確認パラメータ
        /// </summary>
        public const UInt32 CHECK_ISUSB3 = 0xE003180C;

        /// <summary>
        /// SVI05API_Open用定義
        /// </summary>
        public const UInt32 APP_WHO_REC = 1;
        public const UInt32 APP_WHO_CTL = 2;

        /// <summary>
        /// 制御ライブラリの内部変数を初期化します。
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_Init")]
        public static extern void Init();

        /// <summary>
        /// 制御ライブラリの終了処理を行います。
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_End")]
        public static extern void End();

        /// <summary>
        /// デバイスドライバをオープンします。
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_Open")]
        public static extern UInt32 Open(UInt32 ulAppWho, int deviceIndex);

        /// <summary>
        /// デバイスドライバをDeviceID, BoardTypeを指定してオープンします。
        /// DeviceID: 0から連番。BoardType: 0->SVM, 1->SVO
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_OpenEx")]
        public static extern UInt32 OpenEx(UInt32 ulAppWho, int deviceIndex, int boardType);
    
        /// <summary>
        /// Board Name, Board IDを取得します
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_GetBoardInfo")]
        public static extern UInt32 GetBoardInfo(int boardType, int choosedDeviceIndex, StringBuilder boardInfo,  uint boardInfoSize, IntPtr pBoardID);

        /// <summary>
        /// SVM/SVOボードを再起動します
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_RestartSVM")]
        public static extern UInt32 Restart();

        /// <summary>
        /// デバイスドライバをクローズします。
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_Close")]
        public static extern UInt32 Close(UInt32 ulAppWho);

        /// <summary>
        /// SVI05API.DLLのバージョン情報を取得します。
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_GetVersion")]
        //public static extern UInt32 GetVersion(byte[] pucVerBuf);
        public static extern UInt32 GetVersion([MarshalAs(UnmanagedType.LPStr)] StringBuilder pcVerBuf);

        /// <summary>
        /// FPGAレジスタを読み込みます。
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_SPIFpgaRead")]
        public static extern UInt32 FpgaRegisterRead(byte ucCommdId, byte ucSSId, UInt32 ulAddress, UInt32 ulLen, IntPtr pucRcvBuf);

        /// <summary>
        /// FPGAレジスタを書き込みます。
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_SPIFpgaWrite")]
        public static extern UInt32 FpgaRegisterWrite(byte ucCommdId, byte ucSSId, UInt32 ulAddress, UInt32 ulLen, IntPtr pucSendBuf);

        /// <summary>
        /// I2Cで1ブロック受信します。RestartConditionが有効な状態で動作します。
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_I2CBlockRead")]
        public static extern UInt32 I2CBlockRead(UInt32 ulSlaveAdr, UInt32 ulSubAdr, UInt32 ulLen, IntPtr pucRcvBuf, int sAddrType);

        /// <summary>
        /// I2Cで1ブロック受信します。ulReadModeでRestartConditionの有効/無効が切り替えできます。
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_I2COneBlockRead")]
        public static extern UInt32 I2COneBlockRead(UInt32 ulSlaveAdr, UInt32 ulLen, UInt32 ulReadMode, IntPtr pucRcvBuf);

        /// <summary>
        /// I2Cで1ブロック送信します。RestartConditionが有効な状態で動作します。
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_I2CBlockWrite")]
        public static extern UInt32 I2CBlockWrite(UInt32 ulSlaveAdr, UInt32 ulSubAdr, UInt32 ulLen, IntPtr pucSendBuf, int sAddrType);

        /// <summary>
        /// I2Cで1ブロック送信します。ulWriteModeでRestartConditionの有効/無効が切り替えできます。
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_I2COneBlockWrite")]
        public static extern UInt32 I2COneBlockWrite(UInt32 ulSlaveAdr, UInt32 ulLen, UInt32 ulWriteMode, IntPtr pucSendBuf);

        /// <summary>
        /// FX3とFPGAのバージョン情報を取得します。
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_SVMVersionInfo")]
        public static extern UInt32 SVMVersionInfo(UInt32 ulLen, IntPtr pucRcvBuf);

        /// <summary>
        /// SVボードの内部分類番号を返却します
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_GetRevision")]
        public static extern UInt32 SVI05API_GetRevision(IntPtr svm_03_revision, IntPtr versionInfo, IntPtr customCode);

        /// <summary>
        /// SPI Read命令を実行します。
        /// </summary>
        [DllImport(dllName, EntryPoint = "SVI05API_SPIRead")]
        public static extern UInt32 SPIRead(byte ucCommdId, byte ucSSId, UInt32 ulAddress, UInt32 ulLen, IntPtr pucRcvBuf);
    }
}
